@@ -1,33 +1,37 @@
 package Perinci::Sub::Gen::AccessTable::DBI;
 
+our $DATE = '2015-02-18'; # DATE
+our $VERSION = '0.12'; # VERSION
+
 use 5.010001;
 use strict;
 use warnings;
 use experimental 'smartmatch';
 use Log::Any '$log';
 
+use Function::Fallback::CoreOrPP qw(clone);
 use Locale::TextDomain::UTF8 'Perinci-Sub-Gen-AccessTable-DBI';
-use Data::Clone;
 use DBI;
 use Perinci::Sub::Gen::AccessTable qw(gen_read_table_func);
+use Perinci::Sub::Util qw(gen_modified_sub);
 
 require Exporter;
 our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw(gen_read_dbi_table_func);
 
-our $VERSION = '0.11'; # VERSION
-
-our %SPEC;
 my $label = "(gen_read_dbi_table_func)";
 
 sub __parse_schema {
-    require Data::Sah;
-    Data::Sah::normalize_schema($_[0]);
+    require Data::Sah::Normalize;
+    Data::Sah::Normalize::normalize_schema($_[0]);
 }
 
-my $spec = clone $Perinci::Sub::Gen::AccessTable::SPEC{gen_read_table_func};
-$spec->{summary} = 'Generate function (and its metadata) to read DBI table';
-$spec->{description} = <<'_';
+gen_modified_sub(
+    output_name => 'gen_read_dbi_table_func',
+    install_sub => 0,
+    base_name   => 'Perinci::Sub::Gen::AccessTable::gen_read_table_func',
+    summary     => 'Generate function (and its metadata) to read DBI table',
+    description => <<'_',
 
 The generated function acts like a simple single table SQL SELECT query,
 featuring filtering, ordering, and paging, but using arguments as the 'query
@@ -36,13 +40,22 @@ API function. Please see Perinci::Sub::Gen::AccessTable's documentation for more
 details on what arguments the generated function will accept.
 
 _
-delete $spec->{args}{table_data};
-$spec->{args}{table_name} = {
-    req => 1,
-    schema => 'str*',
-    summary => 'DBI table name',
-};
-$spec->{args}{table_spec}{description} = <<'_';
+    remove_args => ['table_data'],
+    add_args    => {
+        table_name => {
+            req => 1,
+            schema => 'str*',
+            summary => 'DBI table name',
+        },
+        dbh => {
+            schema => 'obj*',
+            summary => 'DBI database handle',
+        },
+    },
+    modify_args => {
+        table_spec => sub {
+            my $as = shift;
+            $as->{description} = <<'_';
 
 Just like Perinci::Sub::Gen::AccessTable's table_spec, except that each field
 specification can have a key called `db_field` to specify the database field (if
@@ -50,11 +63,9 @@ different). Currently this is required. Future version will be able to generate
 table_spec from table schema if table_spec is not specified.
 
 _
-$spec->{args}{dbh} = {
-    schema => 'obj*',
-    summary => 'DBI database handle',
-};
-$SPEC{gen_read_dbi_table_func} = $spec;
+        },
+    },
+);
 sub gen_read_dbi_table_func {
     my %args = @_;
 
@@ -190,7 +201,7 @@ sub gen_read_dbi_table_func {
 }
 
 1;
-# ABSTRACT: Generate function (and its Rinci metadata) to access DBI table data
+# ABSTRACT: Generate function (and its metadata) to read DBI table
 
 __END__
 
@@ -200,11 +211,11 @@ __END__
 
 =head1 NAME
 
-Perinci::Sub::Gen::AccessTable::DBI - Generate function (and its Rinci metadata) to access DBI table data
+Perinci::Sub::Gen::AccessTable::DBI - Generate function (and its metadata) to read DBI table
 
 =head1 VERSION
 
-This document describes version 0.11 of Perinci::Sub::Gen::AccessTable::DBI (from Perl distribution Perinci-Sub-Gen-AccessTable-DBI), released on 2014-05-27.
+This document describes version 0.12 of Perinci::Sub::Gen::AccessTable::DBI (from Perl distribution Perinci-Sub-Gen-AccessTable-DBI), released on 2015-02-18.
 
 =head1 SYNOPSIS
 
@@ -357,8 +368,8 @@ Specify defaults for generated function's arguments.
 
 Can be used to supply default filters, e.g.
 
-    # limit years for credit card expiration date
-    { "year.min" => $curyear, "year.max" => $curyear+10, }
+ # limit years for credit card expiration date
+ { "year.min" => $curyear, "year.max" => $curyear+10, }
 
 =item * B<default_detail> => I<bool>
 
@@ -447,28 +458,28 @@ By default, generated function will be installed to the specified (or caller's)
 package, as well as its generated metadata into %SPEC. Set this argument to
 false to skip installing.
 
-=item * B<langs> => I<array> (default: ["en_US"])
+=item * B<langs> => I<array[str]> (default: ["en_US"])
 
 Choose language for function metadata.
 
 This function can generate metadata containing text from one or more languages.
-For example if you set 'langs' to ['enI<US', 'id>ID'] then the generated function
+For example if you set 'langs' to ['en_US', 'id_ID'] then the generated function
 metadata might look something like this:
 
-    {
-        v => 1.1,
-        args => {
-            random => {
-                summary => 'Random order of results', # English
-                "summary.alt.lang.id_ID" => "Acak urutan hasil", # Indonesian
-                ...
-            },
-            ...
-        },
-        ...
-    }
+ {
+     v => 1.1,
+     args => {
+         random => {
+             summary => 'Random order of results', # English
+             "summary.alt.lang.id_ID" => "Acak urutan hasil", # Indonesian
+             ...
+         },
+         ...
+     },
+     ...
+ }
 
-=item * B<name>* => I<str>
+=item * B<name> => I<str>
 
 Generated function's name, e.g. `myfunc`.
 
@@ -493,24 +504,22 @@ DBI table name.
 
 Table specification.
 
-Just like Perinci::Sub::Gen::AccessTable's tableI<spec, except that each field
+Just like Perinci::Sub::Gen::AccessTable's table_spec, except that each field
 specification can have a key called C<db_field> to specify the database field (if
 different). Currently this is required. Future version will be able to generate
-table>spec from table schema if table_spec is not specified.
+table_spec from table schema if table_spec is not specified.
 
 =item * B<word_search> => I<bool> (default: 0)
 
 Decide whether generated function will perform word searching instead of string searching.
 
 For example, if search term is 'pine' and field value is 'green pineapple',
-search will match if wordI<search=false, but won't match under word>search.
+search will match if word_search=false, but won't match under word_search.
 
 This will not have effect under 'custom_search'.
 
 =back
 
-Return value:
-
 Returns an enveloped result (an array).
 
 First element (status) is an integer containing HTTP status code
@@ -520,18 +529,12 @@ First element (status) is an integer containing HTTP status code
 element (meta) is called result metadata and is optional, a hash
 that contains extra information.
 
+Return value: A hash containing generated function, metadata (hash)
+
 =head1 CAVEATS
 
 It is often not a good idea to expose your database schema directly as API.
 
-=head1 TODO
-
-=over 4
-
-=item * Generate table_spec from database schema, if unspecified
-
-=back
-
 =head1 FAQ
 
 =head1 SEE ALSO
@@ -544,7 +547,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Perinci-Su
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/sharyanto/perl-Perinci-Sub-Gen-AccessTable-DBI>.
+Source repository is at L<https://github.com/perlancar/perl-Perinci-Sub-Gen-AccessTable-DBI>.
 
 =head1 BUGS
 
@@ -556,11 +559,11 @@ feature.
 
 =head1 AUTHOR
 
-Steven Haryanto <stevenharyanto@gmail.com>
+perlancar <perlancar@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Steven Haryanto.
+This software is copyright (c) 2015 by perlancar@cpan.org.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
